#ifndef _WTYPES_H
#define _WTYPES_H

#include "defines.h"

#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <fcntl.h>
#include <grp.h>
#include <pwd.h>
#include <sys/param.h>  /* fuer MAXPATHLEN ... */
#ifdef HAVE_SYS_STATVFS_H
  #include <sys/statvfs.h>
#endif
#ifdef HAVE_SYS_VFS_H
  #include <sys/vfs.h>
#endif
#ifdef HAVE_SYS_SYSINFO_H
  #include <sys/sysinfo.h>
#endif

#include "COMS.h"

enum {SORT_NAME=0,SORT_SIZE,SORT_ACCESS,SORT_MODIFICATION,SORT_CHANGE};
enum {PISNOTHING,PISBUTTON,PISPATH,PISHOTKEY,PISNUMBER,PISFILETYPE};

#define LISTER_MODES 2
#define KEYSTATEMASK(x) (x&(ControlMask|Mod1Mask|ShiftMask))

#ifndef ARG_MAX
  #ifdef _POSIX_ARG_MAX
    #define ARG_MAX _POSIX_ARG_MAX
  #else
    #define ARG_MAX MAXPATHLEN
  #endif
#endif

#define EXE_STRING_LEN ARG_MAX

typedef char bool;

struct _buttondef { char check;
                    char *text;
                    int fg;
                    int bg;
                    unsigned long shortkey;
                    int mods;
                    long comnr;
                    long flag1;
                    long flag2;
                    char *usercom;
                    struct _buttondef *next; };

struct _pathdef { char check;
                  char *text;
                  int fg;
                  int bg;
                  unsigned long shortkey;
                  int mods;
                  char *path;
                  struct _pathdef *next;
                  struct _pathdef *prev; };

struct _filetypedef { char *name;
                      char *info;
                      long comnr;
                      long flag1;
                      long flag2;
                      char *usercom;
                      char *pattern;
                      short filedesc[64];
                      /* 0-255 fr jeweiligen ASCII-Code, -1 keine Beachtung,
                         -2 Ende der Beschreibung */
                      bool usepattern;
                      bool usefiledesc;
                      struct _filetypedef *next; };

struct _hotkeydef { char *name;
                    unsigned long shortkey;
                    int mods;
                    long comnr;
                    long flag1;
                    long flag2;
                    char *usercom;
                    struct _hotkeydef *next; };

struct _wconfig { int colors;
                  struct _color *first;
                  char *lang;
                  char *font;
                  int statusbarfg;
                  int statusbarbg;
                  int sel_lvbfg;
                  int sel_lvbbg;
                  int unsel_lvbfg;
                  int unsel_lvbbg;
                  int sel_dirfg;
                  int sel_dirbg;
                  int unsel_dirfg;
                  int unsel_dirbg;
                  int sel_filefg;
                  int sel_filebg;
                  int unsel_filefg;
                  int unsel_filebg;
                  int clockbarfg;
                  int clockbarbg;
                  int requestfg;
                  int requestbg;
                  int sel_pathfg;
                  int sel_pathbg;
                  int unsel_pathfg;
                  int unsel_pathbg;
                  bool lefthbar_top;
                  bool leftvbar_left;
                  int lefthbar_height;
                  int leftvbar_width;
                  bool righthbar_top;
                  bool rightvbar_left;
                  int righthbar_height;
                  int rightvbar_width;
                  char numxbuttons;
                  char numybuttons;
                  char *ldir;
                  char *rdir;
                  struct _buttondef *firstbutton;
                  struct _pathdef *firstpath;
                  struct _filetypedef *firsttype;
                  struct _hotkeydef *firsthotkey;
                  };

typedef struct _fileentry { char *fullname;
                            char *name;
                            long size;
                            long lsize;
                            long dirsize;
                            bool select;
                            time_t lastaccess;
                            time_t llastaccess;
                            time_t lastmod;
                            time_t llastmod;
                            time_t lastchange;
                            time_t llastchange;
                            mode_t mode;
                            mode_t lmode;
                            bool islink;
			    bool iscorrupt;
                            uid_t userid;
                            uid_t luserid;
                            gid_t groupid;
                            gid_t lgroupid;
                            struct _filetypedef *filetype;
                            int filetypeindex;
                            long nr;
                            int maxlen;
			    int nameWidth;
			    int sizeWidth;
			    int typeWidth;
                            bool infilter;
                            struct _fileentry *next;
                            struct _fileentry *prev;} _FileEntry;

typedef struct _filter { int check;
                         /* 0 fr nicht benutzen
                            1 fr einschlieen
                            2 fr ausschlieen */
                         char *pattern;
                         struct _filter *next; } _Filter;

typedef struct _modeinfo { void (*prog[COMNR])();
                           void (*internprog[100])();
                           void (*activate)();
                           void (*deactivate)();
                           void (*on)(int);
                           void (*off)(int);
                           void (*shutdown)(void);
			   void (*update)(int);
                           void *special_info;
                           char *name;
                           bool selectable;} _ModeInfo;
                           
typedef struct _reallister { char *Dir[2];
                             _FileEntry *FEDir[2];
                             _FileEntry *nextcheck[2];
                             bool filesunchecked;
                             long dirnr[2][2],filenr[2][2],selbytes[2];
                             _FileEntry *activefe[2];
                             int act_sortmode[2];} _RealLister;

typedef struct _iminfo { bool update[2];  /* kann gesetzt werden, um ein Neulesen zu erzwingen */
                         int filenr[2];   /* Informationen fr letzten Datei (falls gendert wird) */
                         int lastmode[2]; /* beschreibt letzten Anzeigemodus */
                         bool finished[2];/* TRUE wenn lesen fertig */
                         /* Anm.: mit letzten ist der Zustand im letzten Durchlauf der Hauptschleife*/} _IMInfo;

typedef struct _lister { struct _listview *lv;
                         struct _stringgadget *path;
                         struct _button *lvb;
                         int activemode;     /* steht fr derzeitigen Kontrollmechanismus */
                         _Filter *first_filter;
			 bool filter_changed;
                         int sortmode;} _Lister;

typedef struct _mainstruct { _Lister lister[2];
                             struct _solidbutton *statebar;
                             int _ActiveLV;
                             _ModeInfo mode_info[LISTER_MODES];
                             struct _loadsets {int _ActiveLV;
                                               int activemode[2];} loadsets;} _MainStruct;

#define MAJOR 1
#define MINOR 3
#define PATCH 3

#endif

